@@ -2,7 +2,7 @@ package Lingua::TokenParse;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.07';
+$VERSION = '0.08';
 
 # NOTE: The {{{ and }}} things are "editor code fold markers".  They
 # are merely a convenience for people who don't care to scroll through
@@ -114,15 +114,16 @@ sub build_parts {  # {{{
 
 sub build_combinations {  # {{{
     my ($self, $i) = @_;
-    $i = 0 unless defined $i;
+    $i    = 0 unless defined $i;
     $prev = 0 unless defined $prev;
 
     for (@{ $self->parts->[$i] }) {
         # Find the end-position of the stem.
         my $n = $i + length;
 
-        # XXX Ugly mystery-hack:
-        # Yank-off the last two stems found, if we are at an "overlap point".
+        # XXX This is an ugly mystery-hack:
+        # Yank-off the last two stems found, if we are at an
+        # "overlap point".
         splice @new, -2 if $prev > $i;
 
 #print "$_ - i: $i, n: $n, prev: $prev, new: ". @new ."\n";
@@ -131,7 +132,8 @@ sub build_combinations {  # {{{
 
         splice @new, @new, $n, $_;
 
-        push @{ $self->combinations }, join '.', @new if $n == length ($self->word);
+        push @{ $self->combinations }, join '.', @new
+            if $n == length ($self->word);
 
         $self->build_combinations($n);
     }
@@ -149,18 +151,7 @@ sub build_knowns {  # {{{
 
         for (@chunks) {
             # Handle hyphens in lexicon entries.
-            my $flag = 0;
-            if (exists $self->lexicon->{$_}) {
-                $flag++;
-            }
-            elsif (exists $self->lexicon->{"$_-"}) {
-                $flag++;
-                $_ = "$_-";
-            }
-            elsif (exists $self->lexicon->{"-$_"}) {
-                $flag++;
-                $_ = "-$_";
-            }
+            ($_, my $flag) = _hyphenate($_, $self->lexicon, 0);
 
             # Sum the combination familiarity value.
             $sum++ if $flag;
@@ -177,27 +168,12 @@ sub build_knowns {  # {{{
 
 sub build_definitions {  # {{{
     my $self = shift;
-
-    # Make a familiar combination from each "raw" combination.
+    # Save combination entries with their definitions as the values.
     for my $combo (keys %{ $self->knowns }) {
-        # Get the bits of the combination.
-        my @chunks = split /\./, $combo;
-
-        # Save entries with their definitions as the values.
-        for (@chunks) {
-            # Handle hyphens in lexicon entries.
-            if (exists $self->lexicon->{$_}) {
-                $self->definitions->{$_} = $self->lexicon->{$_};
-            }
-            elsif (exists $self->lexicon->{"$_-"}) {
-                $self->definitions->{$_} = $self->lexicon->{"$_-"};
-            }
-            elsif (exists $self->lexicon->{"-$_"}) {
-                $self->definitions->{$_} = $self->lexicon->{"-$_"};
-            }
-            else {
-                $self->definitions->{$_} = undef;
-            }
+        for (split /\./, $combo) {
+            $self->definitions->{$_} =
+                exists $self->lexicon->{$_}
+                    ? $self->lexicon->{$_} : undef
         }
     }
 }  # }}}
@@ -217,15 +193,8 @@ sub trim_knowns {  # {{{
         # Concatinate adjacent unknowns.
         for (@chunks) {
             if (defined $self->definitions->{$_}) {
-                if ($unknown) {
-                    if (exists $self->lexicon->{"-$unknown"}) {
-                        $unknown = "-$unknown";
-                    }
-                    elsif (exists $self->lexicon->{"$unknown-"}) {
-                        $unknown = "$unknown-";
-                    }
-                    push @seen, $unknown;
-                }
+                push @seen, scalar _hyphenate($unknown, $self->lexicon)
+                    if $unknown;
                 push @seen, $_;
                 $unknown = '';
             }
@@ -233,50 +202,74 @@ sub trim_knowns {  # {{{
                 $unknown = $unknown . $_;
             }
         }
-        if ($unknown) {
-            if (exists $self->lexicon->{"-$unknown"}) {
-                $unknown = "-$unknown";
-            }
-            elsif (exists $self->lexicon->{"$unknown-"}) {
-                $unknown = "$unknown-";
-            }
-            push @seen, $unknown;
-        }
+        push @seen, scalar _hyphenate($unknown, $self->lexicon)
+            if $unknown;
 
         $combo = join '.', @seen;
         $trimmed{$combo} = $self->knowns->{$combo};
     }
 
     # Delete trimmed combinations that have defined lexicon entries
-    # in the unknowns.
+    # embedded in the unknowns.
     for my $combo (sort keys %trimmed) {
+        # Initialize the "bogus combination flag" for use with the
+        # index() function.
         my $flag = -1;
 
         # Inspect each combination chunk.
         for my $chunk (split /\./, $combo) {
             next if $self->definitions->{$chunk};
 
+            # Loops through the defined fragments.
             # Does the unknown chunk contain a defined lexicon entry?
             for my $entry (
                 grep { defined $self->definitions->{$_} }
                     sort keys %{ $self->definitions }
             ) {
+                # Strip off the stem-hyphen, if it exists.
                 $entry =~ s/-//;
+
+                # Flag the combination as bogus, if it has an
+                # unknown bit that contains a known fragment.
                 $flag = index $chunk, $entry;
+
+                # Bail out, if we found a bogus combination.
                 last if $flag >= 0;
             }
 
+            # Bail out, if we found a bogus combination.
             last if $flag >= 0;
         }
 
+        # Remove combinations that were flagged as being bogus.
         delete $trimmed{$combo} if $flag >= 0;
     }
 
+    # Set the knowns list to the new trimmed list.
     $self->knowns(\%trimmed);
 }  # }}}
 
+sub _hyphenate {  # {{{
+    my ($string, $lex, $flag) = @_;
+
+    if (exists $lex->{$string}) {
+        $flag++ if defined $flag;
+    }
+    elsif (exists $lex->{"-$string"}) {
+        $flag++ if defined $flag;
+        $string = "-$string";
+    }
+    elsif (exists $lex->{"$string-"}) {
+        $flag++ if defined $flag;
+        $string = "$string-";
+    }
+
+    return wantarray ? ($string, $flag) : $string;
+}  # }}}
+
 sub output_knowns {  # {{{
     my $self = shift;
+    my @out = ();
 
     for (reverse
              sort { $self->knowns->{$a} <=> $self->knowns->{$b} }
@@ -289,8 +282,12 @@ sub output_knowns {  # {{{
                     ? $self->definitions->{$chunk}
                     : '?';
         }
-        printf qq/%s: %0.2f\n"%s"\n\n/, $_, $self->knowns->{$_}, join '; ', @definition;
+
+        push @out, sprintf qq/%s: %0.2f\n"%s"/,
+            $_, $self->knowns->{$_}, join '; ', @definition;
     }
+
+    return wantarray ? @out : join "\n\n", @out;
 }  # }}}
 
 1;
@@ -308,10 +305,10 @@ Lingua::TokenParse - Parse a word into scored, fragment combinations
   my %lexicon;
   @lexicon{qw(ti art ion)} = qw(foo bar baz);
   my $obj = Lingua::TokenParse->new(
-      word => $word,
+      word    => $word,
       lexicon => \%lexicon,
   );
-  $obj->output_knowns;
+  print scalar $obj->output_knowns;
 
   # Okay.  Now, let's parse a new word.
   $obj->word('metaphysical');
@@ -323,7 +320,7 @@ Lingua::TokenParse - Parse a word into scored, fragment combinations
       '-al'   => 'relating to, characterized by',
   });
   $obj->parse;
-  $obj->output_knowns;
+  my @knowns = $obj->output_knowns;
 
 =head1 ABSTRACT
 
@@ -363,15 +360,6 @@ Return a new Lingua::TokenParse object.
 This method will automatically call the partition methods (detailed 
 below) if a word and lexicon are provided.
 
-=head2 _reset_parse()
-
-  $obj->_reset_parse();
-
-Reset the lists used to parse a word into fragment combinations.
-
-This method is automatically called prior to reparsing a new word in 
-the same session and should never need to be called explicitly.
-
 =head2 parse()
 
   $obj->parse();
@@ -425,10 +413,15 @@ unknown fragments concatinated.
 
 =head2 output_knowns()
 
-  $obj->output_knowns();
+  print $obj->output_knowns();
 
-Convenience method to output the familiar word part combinations with
-their familiarity scores rounded to two decimals.
+Convenience method to return the familiar word part combinations with
+their familiarity scores (rounded to two decimals) and semicolon 
+separated, fragment definitions in either scalar or array context.
+
+In scalar context, a single, newline separated string is returned.
+In array context, each of these combinations is a separate entry in 
+an array.
 
 =head1 ACCESSORS
 
@@ -504,8 +497,6 @@ improvements!
 Calculate familiarity with more granularity.  Possibly with a
 multidimensional measure.
 
-Output some type of concatinated, known combination definition.
-
 Handle the build_combinations method and related globals correctly.
 
 Compute the time required for a given parse.
